/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2009                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include<vcg/simplex/vertex/base.h>
#include<vcg/simplex/vertex/component.h>

#include <vcg/complex/used_types.h>

#include<vcg/simplex/face/base.h>
#include<vcg/simplex/face/component.h>
#include<vcg/complex/complex.h>
#include<vcg/complex/algorithms/create/platonic.h>
#include<wrap/io_trimesh/import.h>

using namespace vcg;
using namespace std;

class MyEdge;
class MyFace;
class MyVertex;
struct MyUsedTypes : public UsedTypes<	Use<MyVertex>   ::AsVertexType,
                                        Use<MyEdge>     ::AsEdgeType,
                                        Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex<MyUsedTypes, vertex::Coord3f, vertex::Normal3f, vertex::BitFlags  >{};
class MyFace    : public Face< MyUsedTypes, face::VertexRef, face::BitFlags > {};
class MyEdge    : public Edge<MyUsedTypes>{};
class MyMesh    : public tri::TriMesh< vector<MyVertex>, vector<MyFace> , vector<MyEdge>  > {};


class fakeEdge
{
public:
  MyVertex * v0;
  MyVertex * v1;

  fakeEdge(MyVertex * _v0,MyVertex * _v1)
  {
    v0=_v0;
    v1=_v1;
    if(v0>v1) std::swap(v0,v1);
  }
  bool operator < (const fakeEdge &other) const
  {
    if(this->v0!=other.v0) return this->v0< other.v0;
    return this->v1< other.v1;
  }
  bool operator == (const fakeEdge &other) const
  {
    return (this->v0 == other.v0)&& (this->v1 == other.v1);
  }
};

int main( int argc, char **argv )
{
  MyMesh m;
  if(argc < 1 ) return -1;

  printf("Reading %s\n",argv[1]);
//  tri::Tetrahedron(m);
  tri::io::ImporterPLY<MyMesh>::Open(m,argv[1]);

  printf("Mesh has %i vert and %i faces\n",m.vn, m.fn);

  MyMesh::FaceIterator fi;
  std::set<fakeEdge> edgeSet;
  int nonBorderEdgeNum =0;
  for(fi=m.face.begin(); fi!=m.face.end();++fi)
  {
    for(int i=0;i<3;++i)
    {
      fakeEdge fe(fi->V0(i),fi->V1(i));
      std::set<fakeEdge>::iterator fit = edgeSet.find(fe);
      if(fit==edgeSet.end())
          edgeSet.insert(fe);
      else nonBorderEdgeNum++;
    }
  }


  int edgeNum=edgeSet.size();
  printf("Edgenum = %i\n",edgeNum);
  printf("nonBorderEdgeNum = %i\n",nonBorderEdgeNum);
  printf("BorderEdgeNum = %i\n",edgeNum -nonBorderEdgeNum);

  vector<fakeEdge> edgeVector;
  for(fi=m.face.begin(); fi!=m.face.end();++fi)
  {
    for(int i=0;i<3;++i)
    {
      fakeEdge fe(fi->V0(i),fi->V1(i));
      edgeVector.push_back(fe);
    }
  }

  std::sort(edgeVector.begin(),edgeVector.end());
  int edgeNum2= std::unique(edgeVector.begin(),edgeVector.end())-edgeVector.begin();

  printf("Edgenum = %i\n",edgeNum2);

  printf (" Vn - En + Fn = %i\n",m.vn-edgeNum+m.fn);

  return 0;
}
